/*
 * The MIT License (MIT)

Copyright (c) 2013 Empscraft.com / GorillaFresh

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/


/*
 * Many thanks to https://github.com/xauth/xauth's project authors for providing the base idea and some code references in this class!
 */


package com.empscraft.bouncer;

import org.bukkit.Location;
import org.bukkit.entity.Entity;
import org.bukkit.entity.HumanEntity;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.EntityRegainHealthEvent;
import org.bukkit.event.entity.EntityTargetEvent;
import org.bukkit.event.entity.FoodLevelChangeEvent;
import org.bukkit.event.entity.PotionSplashEvent;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.player.AsyncPlayerChatEvent;
import org.bukkit.event.player.PlayerDropItemEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerMoveEvent;
import org.bukkit.event.player.PlayerPickupItemEvent;
import org.bukkit.event.player.PlayerQuitEvent;

import com.empscraft.bouncer.model.Session;


public class BouncerListener implements Listener {
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onBlockBreak(BlockBreakEvent event) {
		if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified(event.getPlayer())) {
			event.getPlayer().sendMessage(Bouncer.getSingleton().playerAssistant.notVerified);
			Bouncer.getSingleton().playerAssistant.cantMessage(event.getPlayer());
			event.setCancelled(true);
		}
	}

	@EventHandler(priority = EventPriority.LOWEST)
	public void onBlockPlace(BlockPlaceEvent event) {
		if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified(event.getPlayer())) {
			Bouncer.getSingleton().playerAssistant.cantMessage(event.getPlayer());
			event.setCancelled(true);
		}
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onEntityDamage(EntityDamageEvent event) {
		Entity entity = event.getEntity();
		if (entity instanceof Player && ((Player)entity).isOnline()) { // player taking damage
			if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player)entity)) {
				event.setCancelled(true);
			}
		}

		if (event instanceof EntityDamageByEntityEvent) { // player dealing damage to other entity
			EntityDamageByEntityEvent edbeEvent = (EntityDamageByEntityEvent) event;
			Entity damager = edbeEvent.getDamager();
			if (damager instanceof Player) {
				if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player)damager)) {
					Bouncer.getSingleton().playerAssistant.cantMessage((Player)damager);
					event.setCancelled(true);
				}
			}
		}
	}

	@EventHandler(priority = EventPriority.LOWEST)
	public void onEntityTarget(EntityTargetEvent event) {
		Entity target = event.getTarget();
		if (target instanceof Player) {
			if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player)target)) {
				Bouncer.getSingleton().playerAssistant.cantMessage((Player)target);
				event.setCancelled(true);
			}
		}
	}

	@EventHandler(priority = EventPriority.LOWEST)
	public void onFoodLevelChange(FoodLevelChangeEvent event) {
		Entity entity = event.getEntity();
		if (entity instanceof Player) {
			if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player)entity)) {
				event.setCancelled(true);
			}
		}
	}

	@EventHandler(priority = EventPriority.LOWEST)
	public void onPotionSplash(PotionSplashEvent event) {
		for (LivingEntity entity : event.getAffectedEntities()) {
			if (entity instanceof Player) {
				if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player)entity)) {
					event.setIntensity(entity, 0);
				}
			}
		}
	}

	@EventHandler(priority = EventPriority.LOWEST)
	public void onEntityRegainHealth(EntityRegainHealthEvent event) {
		Entity entity = event.getEntity();
		if (entity instanceof Player) {
			if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player) event.getEntity())) {
				event.setCancelled(true);
			}
		}
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onPlayerInteract(PlayerInteractEvent event) {
		if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player) event.getPlayer())) {
			event.setCancelled(true);
		}
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onPlayerMove(PlayerMoveEvent event) {
		if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player) event.getPlayer())) {
			Player p = (Player) event.getPlayer();
			Location oldLoc = event.getFrom();
			Location newLoc = event.getTo();
			if(newLoc.getBlockX() != oldLoc.getBlockX() || newLoc.getBlockY() != oldLoc.getBlockY() || newLoc.getBlockZ() != oldLoc.getBlockZ()) {
				p.teleport(oldLoc);
				Bouncer.getSingleton().playerAssistant.cantMessage((Player) event.getPlayer());
				event.setCancelled(true);
			}
		}
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onPlayerPickupItem(PlayerPickupItemEvent event) {
		if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player) event.getPlayer())) {
			event.setCancelled(true);
		}
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onInventoryClick(final InventoryClickEvent event) {
		HumanEntity entity = event.getWhoClicked();
		if (entity instanceof Player) {
			if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player) entity)) {
				Bouncer.getSingleton().playerAssistant.cantMessage((Player) entity);
				event.setCancelled(true);
			}
		}
	}

	@EventHandler(priority = EventPriority.LOWEST)
	public void onPlayerDropItem(final PlayerDropItemEvent event) {
		if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified((Player) event.getPlayer())) {
			Bouncer.getSingleton().playerAssistant.cantMessage((Player) event.getPlayer());
			event.setCancelled(true);
		}
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onPlayerJoin(final PlayerJoinEvent event) {
		Player p = event.getPlayer();
		if (p == null || !p.isOnline())
			return;
		
		p = Bouncer.getSingleton().playerAssistant.sessionManager.getTruePlayerObject(p);
		
		if(Bouncer.getSingleton().playerAssistant.sessionManager.antiBrute.isLockedOut(p.getAddress().getHostName())) {
			if(Bouncer.getSingleton().playerAssistant.sessionManager.antiBrute.isExpired(p.getAddress().getHostName())) {
				Bouncer.getSingleton().playerAssistant.sessionManager.antiBrute.resetAttempt(p.getAddress().getHostName());
			} else {
				p.kickPlayer("Temporary lockout due to many failed login attempts.");
				return;
			}
		}
		if(!Bouncer.getSingleton().playerAssistant.sessionManager.sessionExists(p)) {
			Session session = new Session(p);
			if(Bouncer.getSingleton().playerAssistant.sessionManager.authAdaptor.usernameExists(p.getName())) {
				session.status = Session.State.Registered;
			} else {
				session.status = Session.State.Guest;
			}
			Bouncer.getSingleton().playerAssistant.sessionManager.addSession(p, session);
		} else {
			p.kickPlayer("[Err]");
		}
		p.sendMessage("This server runs Bouncer for forum bridging for http://empscraft.com/forum");
		p.sendMessage("If you have an account /verify [password] to play");
		p.sendMessage("Otherwise you can /register [password] or /register [password] [email] to make an account.");
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onPlayerQuit(final PlayerQuitEvent event) {
		Bouncer.getSingleton().playerAssistant.sessionManager.playerSession.remove(Bouncer.getSingleton().playerAssistant.sessionManager.getTruePlayerObject(event.getPlayer()));
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onAsyncPlayerChat(AsyncPlayerChatEvent event) {
		if(!Bouncer.getSingleton().playerAssistant.sessionManager.isVerified(event.getPlayer())) {
			Bouncer.getSingleton().playerAssistant.cantMessage((Player) event.getPlayer());
			event.setCancelled(true);
		}
	}
	
}
