/*
 * The MIT License (MIT)

Copyright (c) 2013 Empscraft.com / GorillaFresh

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package com.empscraft.bouncer.auth.hash;

import java.math.BigInteger;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import com.empscraft.bouncer.model.Hash;


public class SHA1 implements Hash {

	public String compute(String s) {
		return sha1(s);
	}
	
	private static final Charset CHARSET_UTF8 = Charset.forName("UTF-8");
	private static final MessageDigest DIGEST_SHA1;

	static {
		try {
			DIGEST_SHA1 = MessageDigest.getInstance("SHA1");
		} catch (NoSuchAlgorithmException ex) {
			throw new ExceptionInInitializerError(ex);
		}
	}

	public static String sha1(String str) {
		byte[] bytes = str.getBytes(CHARSET_UTF8);
		synchronized (DIGEST_SHA1) {
			DIGEST_SHA1.reset();
			bytes = DIGEST_SHA1.digest(bytes);
		}
		String hash = new BigInteger(1, bytes).toString(16).toLowerCase();
		while (hash.length() < 40) {
			hash = "0" + hash;
		}
		return hash;
	}

}
